//
//  Copyright (c) 2004 Nat!
//  All rights reserved.
//
//  A Mulle kybernetiK production
//  www.mulle-kybernetik.com
//
#import "HostPingStatus.h"


#define WAIT_SECONDS  2


@implementation HostPingStatus

- (id) initWithHost:(NSHost *) host
{
   [super init];
   
   host_ = [host retain];
   
   return( self);
}


- (void) dealloc
{
   [host_ release];
   [lock_ release];
   
   [super dealloc];
}


//
// clumsy, but we need suid anyway for ICMP packets
// and 
//
- (BOOL) ping
{
   NSTask              *task;
   NSPipe              *pipeOutput;
   NSPipe              *pipeError;
   NSEnumerator        *rover;
   NSArray             *arguments;
   NSString            *s;
   NSAutoreleasePool   *pool;
   NSFileHandle        *handle;
   NSData              *data;
   NSArray             *lines;
   NSString            *hostName;
   unsigned int        call;
   static unsigned int   gc;
   int                 seconds;
   
   pool = [[NSAutoreleasePool alloc] init];
   
   task = [[[NSTask alloc] init] autorelease];
   [task setLaunchPath: @"/sbin/ping"];

   hostName = [host_ address];
   if( ! hostName)
      hostName = [host_ name];
   
   NSAssert( hostName, @"Host not set");

   arguments = [NSArray arrayWithObjects: @"-q", @"-n", @"-c", @"1", hostName, nil];
   [task setArguments: arguments];

   pipeOutput = [NSPipe pipe];
   pipeError  = [NSPipe pipe];
   
   // get output and junk it
   [task setStandardOutput:pipeOutput];
   [task setStandardError:pipeError];
   
   call = ++gc;
   NSLog( @"ping #%u start %@", call, hostName);
   [task launch];
   
   // wait max. 2 seconds for a reply
#if WAIT_SECONDS   
   seconds = WAIT_SECONDS;
   while( [task isRunning])
   {
      if( ! seconds)
      {
         [task terminate];
         break;
      }
      sleep( 1);
      --seconds;
   }
#endif         
   [task waitUntilExit];
   NSLog( @"ping #%u end %@", call, hostName);

   
   if( [task terminationStatus] == 0)
   {
      handle = [pipeOutput fileHandleForReading];
      data   = [handle readDataToEndOfFile];
      
      s = [[[NSString alloc] initWithData:data
                                 encoding:NSASCIIStringEncoding] 
            autorelease];
      lines   = [s componentsSeparatedByString:@"\n"];
      
      // ok find our line
      rover = [lines objectEnumerator];
      while( s = [rover nextObject])
         if( [s hasPrefix:@"1 packets transmitted"])
            break;
      
      if( s)
         status_ = [s hasSuffix:@"100% packet loss"] ? HostStatusDown : HostStatusUp;
      else
         status_ = HostStatusError;

      NSLog( @"%@ %@ -> %d", hostName, s, status_);
   }
   else
      status_ = HostStatusError;
   
   [pool release];
   
   return( status_ == HostStatusUp);
}


- (void) unlockingPing
{
   [self retain];

   [self ping];
   [lock_ unlock];

   [self release];
}


- (int) status
{
   return( status_);
}


- (BOOL) isUp
{
   return( status_ == HostStatusUp);
}


- (void) setIsUp:(BOOL) flag
{
   status_ = flag ? HostStatusUp : HostStatusDown;
}


- (NSHost *) host
{
   return( host_);
}


- (void) updateAsynchronously
{
   if( ! lock_)
      lock_ = [[NSLock alloc] init];

   [lock_ lock];
   [NSThread detachNewThreadSelector:@selector( unlockingPing)
                            toTarget:self
                          withObject:nil];
}

@end
