//
//  Copyright (c) 2004 Nat!
//  All rights reserved.
//
//  A Mulle kybernetiK production
//  www.mulle-kybernetik.com
//
#import "MulleHostCheckPref.h"


static NSString   *HostNameKey = @"host";
static NSString   *EnabledKey  = @"enabled";


@implementation MulleHostCheckPref

- (id) init
{
   [super init];
   
   hostList = [NSMutableArray new];
   
   return( self);
}


- (void) dealloc
{
   [hostList release];
   [super dealloc];
}


- (NSHost *) hostFromTextField
{
   NSHost    *host;
   NSString  *s;
   
   s = [hostField stringValue];
   host = [NSHost hostWithAddress:s];
   if( ! host)
      host = [NSHost hostWithName:s];

   return( host);
}


- (void) syncButtons
{
   unsigned int   selectedRows;
   int            selectedRowIndex;
   unsigned int   numberOfRows;
   BOOL           flag;
   
   selectedRows     = [tableView numberOfSelectedRows];
   selectedRowIndex = [tableView selectedRow];
   numberOfRows     = [tableView numberOfRows];
   
   flag = NO;
   if( selectedRows)
   {
      NSDictionary   *dictionary;
      
      dictionary = [hostList objectAtIndex:selectedRowIndex];
      flag       = [[dictionary objectForKey:EnabledKey] boolValue];
   }
   
   [addButton setEnabled:[self hostFromTextField] ? YES : NO];
   [removeButton setEnabled:selectedRows];
   [upButton setEnabled:selectedRows && selectedRowIndex != 0];
   [downButton setEnabled:selectedRows && selectedRowIndex != numberOfRows - 1];
   [toggleButton setEnabled:selectedRows];
   [toggleButton setState:flag ? NSOnState : NSOffState];
}


- (void) syncTableView
{
   [tableView reloadData];
}


- (void) sync
{
   [self syncTableView];
   [self syncButtons];
}


static NSString   *UserDefaultsHostListKey = @"com.mulle-kybernetik.software.MulleHostCheck.hostList";

- (void) loadFromUserDefaults
{
   NSArray   *array;

   if( hostList)
      [hostList release];
   
   array    = [[NSUserDefaults standardUserDefaults] arrayForKey:UserDefaultsHostListKey];
   hostList = [[NSMutableArray alloc] initWithArray:array];
}


- (void) notify
{
   [[NSDistributedNotificationCenter defaultCenter] postNotificationName:@"com.mulle-kybernetik.software.MulleHostCheck.preferencesChanged"
                                                                  object:nil 
                                                                userInfo:nil
                                                      deliverImmediately:NO];
}


- (void) saveToUserDefaults
{
   NSUserDefaults   *defaults;
   
   defaults = [NSUserDefaults standardUserDefaults];
   [defaults setObject:hostList
                forKey:UserDefaultsHostListKey];
   [defaults synchronize];
   
   // send it in a thread
   // sendpref changes
   [NSThread detachNewThreadSelector:@selector( notify)
                            toTarget:self
                          withObject:nil];
   
}


- (void) mainViewDidLoad
{
   [self loadFromUserDefaults];

   [self sync];
}


- (NSDictionary *) dictionaryWithHostName:(NSString *) name
                                  enabled:(BOOL) flag
{
   NSDictionary   *dictionary;

   dictionary = [NSDictionary dictionaryWithObjectsAndKeys:
      name, HostNameKey,
      [NSNumber numberWithBool:flag], EnabledKey,
      nil];
   return( dictionary);
}


/* button */
- (IBAction) addHost:(id) sender
{
   NSDictionary   *dictionary;
   NSString       *hostName;
   NSHost         *host;
   unsigned int   numberOfRows;
   int            index;
   
   host       = [self hostFromTextField];
   hostName   = [host name];
   if( ! hostName)
      hostName = [host address];
   
   dictionary = [self dictionaryWithHostName:hostName
                                     enabled:YES];

   numberOfRows = [tableView numberOfRows];
   if( numberOfRows)
   {
      index = [tableView selectedRow];
      [hostList insertObject:dictionary
                     atIndex:index + 1];
   }
   else
      [hostList addObject:dictionary];
   [self saveToUserDefaults];

   [hostField setStringValue:@""];
   
   [self sync];
}


- (IBAction) removeHost:(id) sender
{
   [hostList removeObjectAtIndex:[tableView selectedRow]];
   [self saveToUserDefaults];

   [self sync];
}


- (IBAction) upHost:(id) sender
{
   NSDictionary   *dictionary;
   unsigned int   index;
   
   index      = [tableView selectedRow];
   dictionary = [[hostList objectAtIndex:index] retain];
   [hostList removeObjectAtIndex:[tableView selectedRow]];
   [hostList insertObject:dictionary
                  atIndex:index - 1];
   [dictionary release];
   [self saveToUserDefaults];

   [tableView selectRow:index - 1
   byExtendingSelection:NO];

   [self sync];
}


- (IBAction) downHost:(id) sender
{
   NSDictionary   *dictionary;
   unsigned int   index;
   
   index      = [tableView selectedRow];
   dictionary = [[hostList objectAtIndex:index] retain];
   [hostList removeObjectAtIndex:index];
   [hostList insertObject:dictionary
                  atIndex:index + 1];
   [dictionary release];
   [self saveToUserDefaults];
   
   [tableView selectRow:index + 1
   byExtendingSelection:NO];
   [self sync];
}


- (IBAction) toggleEnableHost:(id) sender
{
   NSDictionary   *dictionary;
   unsigned int   index;
   
   index      = [tableView selectedRow];
   dictionary = [hostList objectAtIndex:index];
   dictionary = [self dictionaryWithHostName:[dictionary objectForKey:HostNameKey]
                                     enabled:! [[dictionary objectForKey:EnabledKey] boolValue]];
   
   [hostList removeObjectAtIndex:index];
   [hostList insertObject:dictionary
                  atIndex:index];
   [self saveToUserDefaults];

   [self sync];
}



/* table view */ 
- (id) tableView:(NSTableView *) aTableView
    objectValueForTableColumn:(NSTableColumn *) aTableColumn
            row:(int) rowIndex

{
   NSDictionary   *dictionary;
   NSString       *key;
   id             value;
   
   value = nil;
   key   = [aTableColumn identifier];
   if( key)
   {
      dictionary = [hostList objectAtIndex:rowIndex];
      value      = [dictionary objectForKey:key];
      if( [key isEqualToString:EnabledKey])
         value = [value boolValue] == YES ? @"YES" : @"NO";
   }
   else
      value = [NSNumber numberWithInt:rowIndex + 1];
   
   if( ! value)
      value = @"???";
   
   return( value);
}


- (int) numberOfRowsInTableView:(NSTableView *) aTableView
{
   return( [hostList count]);
}


- (void) tableViewSelectionDidChange:(NSNotification *) aNotification
{
   [self syncButtons];
}


- (void) controlTextDidEndEditing:(NSNotification *) aNotification
{
   NSLog( @"Value textDidChange");
   [self syncButtons];
}

@end
